

;-------------------------------------------------------------------------
;   DInitialize
;	1. Find the IO Base and presentation check
;	2. Direction Check
;	3. EEProm check
;	4. Delay Factor check
;	5. RAM Check
;	6. IRQ Check
;	7. initial internal registers
;
;   Input
;	DS		DGroup
;	ax		= 0  --> Get default value from EEPROM
;			<> 0 --> auto detect
;
;   Output
;	cflag		set on means error
;	ds:ax		error message
;	ds:bx		NPB structure
;	All registers not preserved
;-------------------------------------------------------------------------
public	DInitialize
DInitialize	proc	near
IF    Server_386_driver
	call	InitData
ENDIF

	call	check_ps2
	cli

	mov	EPPValue, FAST_MODE+EXT_MODE

FindIOBase:
IF	NDIS_OS2_WKS_driver or ODI_OS2_WKS_driver
	mov	ax, ds
	mov	es, ax
	lea	di, PrnPortList 		;es:di=printer port list
ELSEIF	  Server_386_driver
	mov	ax, ds
	mov	es, ax
	lea	edi, PrnPortList		;es:di=printer port list
ELSE
	xor	ax, ax
	mov	es, ax
	mov	di, printer			;es:di=printer port list
ENDIF
IFDEF	ps
print 1
ENDIF
	mov	cx, 4				;count of element in list
	call	PresentTest			;check DE620 presentation
	jnc	DetectRDFR
IF	Server_386_driver
	lea	eax, NICNotFound		;error code
ELSE
	lea	ax, NICNotFound 		;error code
ENDIF
	mov	dx, word ptr NICNotFound-2
	jmp	IA_exit
DetectRDFR:
	call	Detect360SL
;loadport
;setport CmdPort
;mov	 al, _EBP NIC_Cmd
;out	 dx, al
	call	Check_Direction

	test	ParamFlag, Bit8
	jz	RDFRFind
	jmp	GetEEPROM			;user specified
RDFRFind:
	call	FindRDFR
	jnc	GetEEPROM

	test	PS2, 4
	jz	FoundError
	cmp	EPPValue, FAST_MODE+EXT_MODE
	jne	FoundError
	mov	EPPValue, EXT_MODE
	jmp	short DetectRDFR
FoundError:
IF	Server_386_driver
	lea	eax, RDFRNotFound		  ;error code
ELSE
	lea	ax, RDFRNotFound		 ;error code
ENDIF
	mov	dx, word ptr RDFRNotFound-2
	jmp	IA_exit
;--------------------
GetEEPROM:
IFDEF	ps
print 3
ENDIF
	mov	al, _EBP NICMode
	push	ax				;save NIC Mode
	mov	_EBP NICMode, NIBBLEMODE
	call	NIC_Mode
;;;	Mode_NIC

	mov	ax, ds
	mov	es, ax
IF	Server_386_driver
	lea	edi, EEBuffer		     ;buffer for EEPROM data
ELSE
	lea	di, EEBuffer		    ;buffer for EEPROM data
ENDIF
	call	ReadEEPROMSetting
	or	ax, ax
	jz	NW6
	call	Prn_Mode
;;;	Mode_Prn
	pop	ax				;restore NIC mode
IF	Server_386_driver
	lea	eax, EEPROMReadFail
ELSE
	lea	ax, EEPROMReadFail
ENDIF
	mov	dx, word ptr EEPROMReadFail-2
	jmp	IA_exit
NW6:
	call	Prn_Mode
;;;	Mode_Prn
	pop	ax				;restore NIC_Mode
	mov	_EBP NICMode, al

	cmp	EEBuffer.Auto, 1		;from EEPROM
	jnz	DetectIRQ

	mov	al, EEBuffer.SCRValue
;	test	al, IRQINV
;	jz	NoInv
;	or	SCRRegister, IRQINV		;set INT LEVEL
NoInv:
	and	al, Bit4+Bit5
;	and	SCRRegister, not Bit4+Bit5
;	or	SCRRegister, al 		;set NICMode
  shr	al, 1
  shr	al, 1
  shr	al, 1
	mov	_EBP NICMode, al

	mov	al, EEBuffer.IRQValue
	mov	_EBP irq, al

	mov	al, EEBuffer.WtDelayFactor
	mov	_EBP DFRRegister, al
	mov	al, EEBuffer.RdDelayFactor
	mov	_EBP RDFRRegister, al

	or	ParamFlag, Bit1+Bit4+Bit5+Bit8+Bit6
;	or	ParamFlag, Bit4+Bit5+Bit8+Bit6
;-------------------------------
public DetectIRQ
DetectIRQ:

	test	ParamFlag, Bit1
	jz	IIQ
	test	ParamFlag, Bit4
	jz	IIQ
	jmp	ModeCheck
IIQ:
	mov	al, _EBP NICMode
	push	ax
	mov	_EBP NICMode, NIBBLEMODE

;@@@@@@ by ted tai
IF    Server_386_driver
ELSE
	call	SetOurVector
ENDIF
	in	al, INT_1IMR
	mov	cl, al				;save temporary
	in	al, 61h
	in	al, 61h
	mov	al, cl
	and	al, 5fh 			;clear bit 7,5
	out	INT_1IMR, al			;enable IRQ 7, 5 to clear IRR

;;	call	Prn_Mode			;return to printer mode
	sti
	in	al, 61h
	in	al, 61h
	in	al, 61h
	in	al, 61h
	mov	al, cl
	cli
	out	INT_1IMR, al			;restore IMR
;----------
	call	NIC_Mode
;;;	Mode_NIC
	loadport
	setport CmdPort
	mov	al, _EBP NIC_Cmd
	or	al, IRQEN
	out	dx, al
IFDEF	ps
print 6
ENDIF
	call	IRQCheck			;IRQ number, IRQ inverse
	jnc	IRQDone
	call	Prn_Mode
;;;	Mode_Prn
	call	RestoreVector
	pop	ax
IF	Server_386_driver
	lea	eax, IRQTestFail
ELSE
	lea	ax, IRQTestFail
ENDIF
	mov	dx, word ptr IRQTestFail-2
	jmp	IA_exit
IRQDone:
	call	Prn_Mode
;;;	Mode_Prn
;;;	call	RestoreVector
	pop	ax
	mov	_EBP NICMode, al
;---------------------
ModeCheck:
;;+ 4/22/1993 Andge Chen		for MPX
	cmp	_EBP MPX_exist, 0
	je	NoMPX
	or	_EBP SCRRegister, IRQINV	;MPX exist, IRQ must inverse
NoMPX:
;;-
	test	ParamFlag, Bit6 		;user specified NIC Mode
	jnz	SetMode
IFDEF	ps
print 2
ENDIF
	call	ModeDetermine
	jnc	TestRAM
	lea	ax, ModeTestFail
	mov	dx, word ptr ModeTestFail-2
	jmp	IA_exit

;--------------------
public SetMode
SetMode:
	mov	al, _EBP NICMode		     ;get
	push	ax				;save
	mov	_EBP NICMode, NIBBLEMODE	     ;Now is Nibble mode
   shl	al, 1
   shl	al, 1
   shl	al, 1
	or	_EBP SCRRegister, al
	call	NIC_Mode			;set to NIC Mode
;;;	Mode_NIC
	mov	al, _EBP SCRRegister		     ;SCR content
;;	  IDC1INIT	  W_SCR
	mov	ah, W_SCR
	call	IDC1INIT
;;+ 4/22/1993 Andge Chen	for MPX must delay
	call	TimeDelay
;;-
	call	Prn_Mode
;;;	Mode_Prn
	pop	ax
	mov	_EBP NICMode, al		 ;restore NIC Mode

;;+ 4/27/1993 Andge Chen	for EPP
	cmp	al, FASTMODE
	jne	NotFAST1
	loadport
	setport CmdPort
	mov	al, IRQEN+INITP
	out	dx, al
NotFAST1:
;;-

;-------
TestRAM:
	mov	al, EEBuffer.RAM_Size	    ;20h=8k, 40h=16k, 80h=32k
	or	al, al				;0=64k
	jz	RAM64K
	shr	al, 1
	shr	al, 1
	jmp	short SaveRAMSize
RAM64K:
	mov	al, 64				;64K RAM
SaveRAMSize:
	mov	ramSize, al

	shl	al, 1
	shl	al, 1				;al=page number
	dec	al				;last page
	mov	_EBP EPRRegister, al		 ;EPR
	mov	_EBP SPRRegister, 12		 ;SPR

	call	NIC_Mode
;;;	Mode_NIC

;----------- set SPR Register ----------------
	mov	al, _EBP SPRRegister
;;	  IDC1INIT	  W_SPR
	mov	ah, W_SPR
	call	IDC1INIT
;----------- set EPR Register ----------------
	mov	al, _EBP EPRRegister
;;	  IDC1INIT	  W_EPR
	mov	ah, W_EPR
	call	IDC1INIT
;--------------------------------------------
;;;	mov	al, EEBuffer.Model	    ;get de620 model
	call	SetEIPValue
;----------- save node ID NDIS spec only -------
	test	ParamFlag, Bit3
	jnz	FindDFR1

	push	bx
IF	Server_386_driver
	mov	ecx, 6
	mov	ebx, 0
	push	ebp
SaveID:
	mov	al, ds:[ebx+EEBuffer].NodeID
	mov	_EBP DNodeAddress, al
	inc	ebx
	inc	ebp
	loop	SaveID
	pop	ebp
ELSE
	mov	cx, 6
	mov	bx, 0
SaveID:
	mov	al, ds:[bx+EEBuffer].NodeID
	mov	[bx+DNodeAddress], al
	inc	bx
	loop	SaveID
ENDIF
	pop	bx
;----------- Find a best Delay factor -------------
FindDFR1:
	test	ParamFlag, Bit5
	mov	al, _EBP DFRRegister
	jnz	SetDFRStart

IFDEF	ps
print 4
ENDIF
	call	FindDFR 			;determine delay factor
	jnc	SetDFRStart

	call	Prn_Mode
;;;	Mode_Prn
IF	Server_386_driver
	lea	eax, DFRTestFail
ELSE
	lea	ax, DFRTestFail
ENDIF
	mov	dx, word ptr DFRTestFail-2
	jmp	IA_exit

SetDFRStart:
	or	al, al
	jz	IDFR
	add	al, 2
IDFR:
	mov	_EBP DFRRegister, al		     ;save delay factor
;;	  IDC1INIT	  W_DFR
;	 mov	 ah, W_DFR
;	 call	 IDC1INIT
	call	WDFRINIT
;----------- RAM TEST -----------------
IFDEF	ps
print 5
ENDIF
RAMVerify:
	sti
	mov	al, ramSize			;get ram size K base
	xor	ah, ah
	mov	cx, 9				;
	shl	ax, cl				;word base
	mov	cx, ax				;word count
	call	RAMTest
int 3
;	jnc	NW7
	jc	gg
	jmp	NW7
gg:
;---------------
; 11/4/1992 Andge Chen
	test	ParamFlag, Bit6 		;user specified NIC Mode
	jz	RAMError
	cmp	_EBP NICMode, FASTMODE
	je	RAMError
	cmp	EPPValue, EXT_MODE
	je	RAMError
	mov	EPPValue, EXT_MODE
	call	Detect360SL
	jmp	RAMVerify
RAMError:
;--------------

	call	Prn_Mode
;;;	Mode_Prn


IF	Server_386_driver
	test	ParamFlag, Bit6
	jz	RRRError
	cmp	_EBP NICMode, NIBBLEMODE
	je	RRRError
	cmp	_EBP NICMode, BYTEMODE
	je	RRRByte
	lea	eax, ModeNotSupport2
	jmp	short RRREnd
RRRByte:
	lea	eax, ModeNotSupport1
	jmp	short RRREnd
RRRError:
	lea	eax, RAMTestFail
RRREnd:
ELSE
	test	ParamFlag, Bit6
	jz	RRRError
	cmp	NICMode, NIBBLEMODE
	je	RRRError
	cmp	NICMode, BYTEMODE
	je	RRRByte
	lea	ax, ModeNotSupport2
	mov	dx, word ptr ModeNotSupport2-2
	jmp	short RRREnd
RRRByte:
	lea	ax, ModeNotSupport1
	mov	dx, word ptr ModeNotSupport1-2
	jmp	short RRREnd
RRRError:
;;+ 4/27/1993 Andge Chen	Auto switch when RAM test error
	cmp	_EBP NICMode, FASTMODE
	jne	CheckB
	call	NIC_Mode
	mov	al, _EBP SCRRegister
	and	al, 0cfh
	or	al, 10h 			;byte mode
	mov	_EBP SCRRegister, al
	mov	ah, W_SCR
	call	IDC1INIT
	call	TimeDelay
	call	Prn_Mode
	mov	_EBP NICMode, BYTEMODE
	jmp	TestRAM
CheckB:
	cmp	_EBP NICMode, BYTEMODE
	jne	NoMoreCheck
	call	NIC_Mode
	mov	al, _EBP SCRRegister
	and	al, 0cfh
	or	al, 00h 			;nibble mode
	mov	_EBP SCRRegister, al
	mov	ah, W_SCR
	call	IDC1INIT
	call	TimeDelay
	call	Prn_Mode
	mov	_EBP NICMode, BYTEMODE
	jmp	TestRAM
NoMoreCheck:
;;-
	lea	ax, RAMTestFail
	mov	dx, word ptr RAMTestFail-2
RRREnd:
ENDIF
	jmp	IA_exit

NW7:
	call	RestoreVector
;----------

InitCard:
	call	NICInitial			;initial adapter
	call	Prn_Mode			;return to printer mode
;loadport
;setport CmdPort
;or	 byte ptr _EBP NIC_Cmd, IRQEN
;mov	 al, _EBP NIC_Cmd
;out	 dx, al

; + modified by Peter Yu on 12/29/92 for MS WFW
IF NDIS_DOS_WKS_driver or ODI_DOS_WKS_driver or Packet_Driver or IPX_WKS_driver

; + 4/17/1993 Andge Chen     for multiplexor
		cmp	MPX_exist, 0
		jne	No_Erase
; - 4/17/1993
		push	es
		push	cx
		push	di
		xor	ax, ax
		mov	es, ax
		mov	cx, 4
		mov	di, 408h
@@:
		mov	ax, word ptr es:[di]
		cmp	ds:[bx].NPB_IOBase, ax
		je	ErasePrinterMem
		add	di, 2
		loop	@B
		jmp	@F
ErasePrinterMem:
		mov	Save_Printer,di 	;
		mov	Save_IOBase,ax		;
		mov	word ptr es:[di], 400h
		mov	di, 411h
		mov	al,byte ptr es:[di]	;
		mov	Save_EQUIP,al		;
		or	byte ptr es:[di], 0c0h
@@:
		pop	di
		pop	cx
		pop	es
No_Erase:
ENDIF
; - modified by Peter Yu on 12/29/92 for MS WFW

IF	Server_386_driver
	lea	ebx, _EBP io_addr
ELSE
	lea	bx, io_addr
ENDIF
	xor	ax, ax
	clc
	ret
IA_exit:
	stc
	ret
DInitialize	endp


;-------------------------------------------------------------------------
; Check_Multiplexer
;-------------------------------------------------------------------------
Check_Multiplexer     proc
	loadport
	setport CmdPort
	mov	al,INITP	; set SLCTIN,INIT,AUTOFD,STROBE to high
	out	dx,al

	in	al,61h

	sub	dx,CmdPort	;data port
	mov	al,CLEAR_MPX+MPX_MODE1
	out	dx,al		; reset pattern compare squence
	slow

	mov	al,017h ;;;97h	       ;
	out	dx,al		; write 1st pattern
	mov	al,057h ;;;d7h	       ;
	out	dx,al		;
	slow

	mov	al,00bh ;;;8bh	       ;
	out	dx,al		; write 2nd pattern
	mov	al,04bh ;;;cbh	       ;
	out	dx,al		;
	slow

	inc	dx		; set DX to STATUS port
	in	al,dx		; read STATUS port value
	and	al,SELECT	; get SELECT bit value
	mov	ah,al		; save AL into AH
	shl	al,1		; shift this bit from Bit4 to Bit5
	xor	al,MPX_CHECK_BIT; inverse this bit value
	or	al,EN_MPX_CHECK+MPX_MODE1	; enable MPX chekc function
	dec	dx		; set DX to DATA port
	out	dx,al
	or	al,CLOCK
	out	dx,al
	slow

	inc	dx
	in	al,dx		; read SELECT bit
	and	al,SELECT	; get SELECT bit value
	xor	al,ah
	jz	cant_find_MPX

	mov	al,ah
	shl	al,1		; shift this bit from Bit4 to Bit5
	or	al,EN_MPX_CHECK+MPX_MODE1	; enable MPX chekc function
	dec	dx		; set DX to DATA port
	out	dx,al
	or	al,CLOCK
	out	dx,al
	slow

	inc	dx
	in	al,dx		; read SELECT bit
	and	al,SELECT	; get SELECT bit value
	xor	al,ah
	jnz	cant_find_MPX

	mov	al,MPX_MODE1	; disable MPX chekc function
	dec	dx		; set DX to DATA port
	out	dx,al
	or	al,CLOCK	; disable MPX check function
	out	dx,al
	slow
	or	al,CLEAR_MPX	; clear MPX pattern check squence
	out	dx,al

	mov	_EBP MPX_exist,1
	clc
	ret

cant_find_MPX:
	mov	_EBP MPX_exist,0
	stc
	ret
Check_Multiplexer     endp


;-------------------------------------------------------------------------
;   ModeDetermine
;	Determine mode type
;
;	Output
;		NICMode
;	Modified Registers
;		ax, bx, cx, dx
;-------------------------------------------------------------------------
ModeDetermine	proc	near
	test	PS2, 4
	jz	NotEPP
	cmp	_EBP io_addr, 03bch
	jz	NotSSL
	mov	ch, FASTMODE
	call	CheckModeType
	jc	NotEPP
	or	_EBP SCRRegister, 20h
	ret
NotEPP:
; 10/8/1992 Andge Chen		for ACER AnyWare NoteBook
	mov	EPPValue, EXT_MODE
	call	Detect360SL
;loadport
;setport CmdPort
;mov	 al, _EBP NIC_Cmd
;out	 dx, al
NotSSL:
	mov	al, _EBP InDirection
	or	al, _EBP OutDirection
	jz	NotByte
	mov	ch, BYTEMODE
	call	CheckModeType
	jc	NotByte
	or	_EBP SCRRegister, 10h
	ret
NotByte:
	mov	ch, NIBBLEMODE
	call	CheckModeType
	mov	_EBP NICMode, NIBBLEMODE
	ret
ModeDetermine	endp


CPRCheck	proc	near
	call	NIC_Mode			;change to NIC mode
;;;	Mode_NIC

	mov	al, 0A5h			;write 0a5h to CPR
;;	  IDC1INIT	  W_CPR
	mov	ah, W_CPR
	call	IDC1INIT

	call	TimeDelay
	loadport
	setport DataPort
	xor	al, al
	out	dx, al
	call	TimeDelay

;;	  IDC2INIT	  R_CPR
	mov	ah, R_CPR
	call	IDC2INIT

	cmp	al, 0A5h			;check return data
	jne	NotEqu
	call	TimeDelay
	call	Prn_Mode
;;;	Mode_Prn
	clc
	ret
NotEqu:
	call	TimeDelay
	call	Prn_Mode
;;;	Mode_Prn
	stc
	ret
CPRCheck	endp

CheckModeType	proc	near
	push	cx
	call	NIC_Mode			;set to NIC Mode
;;;	Mode_NIC
	mov	al, _EBP SCRRegister		     ;SCR content
	and	al, 0cfh
	shl	ch, 1
	shl	ch, 1
	shl	ch, 1
	or	al, ch				;set EPP mode
;;	  IDC1INIT	  W_SCR
	mov	ah, W_SCR
	call	IDC1INIT
;;+ 4/22/1993 Andge Chen	for MPX must delay
	call	TimeDelay
;;-
	call	Prn_Mode
;;;	Mode_Prn
	pop	cx
	mov	_EBP NICMode, ch
;;+ 4/27/1993 Andge Chen	for EPP
	cmp	ch, FASTMODE
	jne	NotFAST2
	loadport
	setport CmdPort
	mov	al, IRQEN+INITP
	out	dx, al
NotFAST2:
;;-
	call	CPRCheck
	jc	ErrorMode
	clc
	ret
ErrorMode:
	call	NIC_Mode
;;;	Mode_NIC
	call	WarmStartProcess
	call	Prn_Mode
;;;	Mode_Prn
	mov	_EBP NICMode, NIBBLEMODE
	stc
	ret
CheckModeType	endp



;-------------------------------------------------------------------------
;   FindRDFR
;	Find a best Read Delay Factor
;
;-------------------------------------------------------------------------
FindRDFR	proc	near
	mov	al, _EBP NICMode
	push	ax				;save mode flag
	mov	_EBP NICMode, NIBBLEMODE	     ;now is nibble mode
nochange:
	call	NIC_Mode
;;;	Mode_NIC

	mov	al, 0A5h			;write 0a5h to CPR
;;	  IDC1INIT	  W_CPR
	mov	ah, W_CPR
	call	IDC1INIT

	mov	_EBP RDFRRegister, 0
CheckLoop:
;	 mov	 al, RDFRRegister
;	 mov	 ah, W_DFR
;	 call	 IDC1INIT
	call	WDFRINIT

	mov	ah, R_CPR
	call	IDC2
	cmp	al, 0a5h
	je	FRDFRRet

	cmp	_EBP RDFRRegister, 20
	jne	IncOneStep
	call	Prn_Mode
;;;	Mode_Prn
	pop	ax
	mov	_EBP NICMode, al
	stc
	ret
IncOneStep:
	inc	byte ptr _EBP RDFRRegister
	jmp	short CheckLoop
FRDFRRet:
	call	Prn_Mode
;;;	Mode_Prn
	pop	ax
	mov	_EBP NICMode, al
	clc
	ret
FindRDFR	endp



;-------------------------------------------------------------------------
;   Detect360SL
;	Check if Intel 82360SL Chip present
;	set to fast mode if 82360sl
;-------------------------------------------------------------------------

;Detect360SL	 proc	 near
;	 cli
;;jmp CheckIOMode
;;jmp found_82360SL
;	 in	 ax,CPUPWRMODE
;mov	 bx, ax
;	 test	 ax,UNLOCK	 ; check CPUPWRMODE reg. locked or not?
;	 jnz	 already_unlock  ; unlock, then jump
;	 mov	 al,0		 ; to enable CPUPWRMODE reg.
;	 out	 CFGSTAT,al
;	 mov	 al,80h
;	 out	 CPUPWRMODE,al
;	 xor	 ah,ah		 ; AX = 0080h
;	 out	 CPUPWRMODE,ax	 ; now CPUPWRMODE reg. enabled
;	 in	 al,61h 	 ; delay
;	 in	 ax,CPUPWRMODE
;	 test	 ax,UNLOCK	 ; check CPUPWRMODE reg. unlocked or not?
;	 jnz	 already_unlock  ; unlock, then jump
;not_SL:
;mov	 ax, bx
;out	 CPUPWRMODE, ax
;	 stc			 ;not found 82360SL
;	 ret
;
;already_unlock:
;;;;;;	 mov	 bx,ax
;;	 and	 ax,not UNIT_ID
;;	 or	 ax,OBM_CNTL+UNIT_EN
;;	 out	 CPUPWRMODE,ax
;in	 al, CPUPWRMODE
;and	 al, 0f0h
;or	 al, 03
;out	 CPUPWRMODE, al
;
;	 mov	 dx,_SIGNATURE
;	 in	 ax,dx
;	 cmp	 ah,43h 			 ;4311h -- version B1
;	 jne	 not_SL 			 ;4301h -- version A1
;
;	 mov	 ah,al
;	 shr	 ah,1
;	 shr	 ah,1
;	 shr	 ah,1
;	 shr	 ah,1		 ; prepare 1st digit of revision number
;	 and	 al,0fh 	 ; prepare 2nd digit of revision number
;	 mov	 byte ptr ver,ah		 ; translate number to version
;	 mov	 byte ptr subver,al		    ; translate number to version
;
;;----------
;mov	 al,0		 ; to enable CPUPWRMODE reg.
;out	 CFGSTAT,al
;mov	 al,80h
;out	 CPUPWRMODE,al
;xor	 ah,ah		 ; AX = 0080h
;out	 CPUPWRMODE,ax	 ; now CPUPWRMODE reg. enabled
;;-----------
;	 mov	 ax,bx
;out	 CPUPWRMODE,ax	 ; Lock CPUPWRMODE register
;test	 ax, UNLOCK
;jnz	 CPU_not_lock
;;	 and	 ax,not UNIT_EN
;;	 out	 CPUPWRMODE,ax	 ; Lock Configuration space
;	 or	 ax,CPUCNFG_LOCK
;	 out	 CPUPWRMODE,ax	 ; Lock CPUPWRMODE register
;CPU_not_lock:
;;	 in	 al,61h
;;;	 in	 ax,CPUPWRMODE
;;;	 test	 ax,UNLOCK+UNIT_EN  ; check CPUPWRMODE reg. locked or not?
;;;	 jnz	 lock_fail
;;-------------
;CheckIOMode:
;	 cmp	 ver, 0 			 ;version A
;	 je	 is_ver_A
;is_ver_B:
;jmp	 found_82360SL
;
;in	 al,CPUPWRMODE
;test	 al,UNLOCK
;jnz	 lock_fail
;mov	 f_CPUPWRMODE,1
;in	 al,CFGSTAT
;test	 al,IOCFGOPN
;jz	 found_82360SL
;;---------
;	 mov	 dx,0fc23h
;	 in	 al,dx
;	 mov	 dx,0f023h
;	 in	 al,dx
;	 mov	 dx,0c023h
;	 in	 al,dx
;	 mov	 dx,00023h
;	 in	 al,dx
;jmp	 found_82360SL
;;	 in	 al,CPUPWRMODE
;;	 test	 al,UNLOCK
;;	 jnz	 lock_fail
;;	 in	 al,CFGSTAT
;;	 test	 al,IOCFGOPN
;;	 jz	 found_82360SL
;
;enable_fail:
;lock_fail:
;	 stc
;	 ret
;
;is_Ver_A:
;	 mov	 dx,0fffbh
;	 in	 al,dx
;	 mov	 dx,0ffffh
;	 in	 al,dx
;	 mov	 dx,0fffdh
;	 in	 al,dx
;	 mov	 dx,0fff9h
;	 in	 al,dx
;	 in	 al,CPUPWRMODE
;	 test	 al,UNLOCK
;	 jnz	 lock_fail
;
;	 in	 al,CFGSTAT
;	 test	 al,IOCFGOPN
;	 jnz	 enable_fail
;
;public found_82360SL
;found_82360SL:
;	 test	 ParamFlag, Bit0	     ;iobase
;	 jnz	 ChkBase
;IF	 NDIS_OS2_WKS_driver or ODI_OS2_WKS_driver or Server_386_driver
;	 mov	 io_addr, 378h
;ELSE
;	 push	 es
;	 mov	 ax, 0
;	 mov	 es, ax
;	 mov	 ax, es:[408h]
;	 pop	 es
;	 mov	 io_addr, ax
;ENDIF
;;;;;	 jmp	 SetFastMode
;ChkBase:
;	 cmp	 io_addr, 3bch
;	 je	 PS2Mode
;	 cmp	 ver, 0 			 ;version A
;	 je	 PS2Mode
;	 cmp	 ver, 1 			 ;version B
;	 jne	 SetFastMode
;	 cmp	 subver, 0			 ;sub version 0
;	 jne	 SetFastMode
;
;PS2Mode:
;	 mov	 al,CFGR2
;	 out	 CFGINDEX,al			 ;select CFGR2
;	 in	 al,CFGDATA
;mov	 dl, al
;	 and	 al, not PS2_EN+SFIO_EN
;	 or	 al, PS2_EN			 ;enable PS2 Mode
;	 mov	 bh,al
;	 mov	 al,CFGR2
;	 out	 CFGINDEX,al
;	 mov	 al,bh
;	 out	 CFGDATA,al			 ;set CFGR2
;
;	 cmp	 io_addr, 0378h
;	 jne	 I278
;	 mov	 al, 0
;	 jmp	 short IOGet
;I278:
;	 cmp	 IO_addr, 0278h
;	 jne	 I3bc
;	 mov	 al, 20h
;	 jmp	 short IOGet
;I3bc:
;	 mov	 al, 40h
;IOGet:
;	 mov	 dx,PORT102
;	 or	 al, DIR_EN			 ;enable bidirection
;	 out	 dx, al
;
;	 test	 ParamFlag, Bit6
;	 jnz	 ModeSet
;	 mov	 NICMode, ByteMode	     ;nic mode
;	 or	 ParamFlag, Bit6
;	 jmp	 short ModeSet
;
;SetFastMode:
;	 mov	 al,CFGR2
;	 out	 CFGINDEX,al			 ;select CFGR2
;	 in	 al,CFGDATA
;mov	 dl,al
;	 and	 al, not PS2_EN+SFIO_EN
;	 or	 al, SFIO_EN			 ;enable PS2 Mode
;	 mov	 bh,al
;	 mov	 al,CFGR2
;	 out	 CFGINDEX,al
;	 mov	 al,bh
;	 out	 CFGDATA,al			 ;set CFGR2
;
;	 out	 SFSENABLE, al			 ;dummy write
;;	 jmp	 $+2
;;	 jmp	 $+2
;
;	 mov	 bh, 10h			 ;select 378h IRQ7
;	 cmp	 io_addr, 378h
;	 je	 SFM1
;	 mov	 bh, 20h			 ;select 278h IRQ5
;SFM1:
;;	  or	  bhal, FAST_MODE+EXT_MODE	    ;enable fast extended mode
;	 or	 bh, FAST_MODE			 ;enable fast extended mode
;	 mov	 al, FPP_CNTL
;	 out	 SFINDEX, al			 ;select FPP_CNTL
;	 mov	 al, bh
;	 out	 SFDATA, al
;	 cmp	 byte ptr ver, 0		 ;check version A
;	 je	 exit0
;
;	 mov	 ah, al
;	 mov	 al, FPP_CNTL
;	 out	 SFINDEX, al
;;	  in	  al, 61h
;;	  in	  al, 61h
;	 in	 al, SFDATA
;	 cmp	 al, ah
;	 jne	 config_failure
;exit0:
;	 test	 ParamFlag, Bit6
;	 jnz	 ModeSet
;	 mov	 NICMode, FastMode	     ;nic mode
;	 or	 ParamFlag, Bit6
;ModeSet:
;	 mov	 int_level, 0		     ;normal
;	 or	 ParamFlag, Bit4	     ;INTLevel
;
;	 or	 ParamFlag, Bit0	     ;IOBase
;
;;	  cmp	  io_addr, 378h
;;	  jne	  IIRQ2
;;	  or	  ParamFlag, Bit1		  ;IRQ
;;	  mov	  irq, 7
;;	  jmp	  SFM2
;;IIRQ2:
;;	  cmp	  io_addr, 278h
;;	  jne	  SFM2
;;	  mov	  irq, 5
;;	  or	  ParamFlag, Bit1		  ;IRQ
;
;SFM2:
;	 or	 PS2, 4 			 ;indicate 360SL
;
;mov	 al,CFGR2
;out	 CFGINDEX,al
;mov	 al,dl
;out	 CFGDATA,al
;
;cmp	 f_CPUPWRMODE,0
;jnz	 not_lock_360SL
;
;	 mov	 al,IDXLCK
;	 out	 CFGINDEX,al
;	 mov	 al,LOCK360SL
;	 out	 CFGDATA,al
;not_lock_360SL:
;	 clc
;	 ret
;
;config_failure:
;	 stc
;	 ret
;
;Detect360SL	 endp

Detect360SL	proc	near
;;+ 5/13/1993 Andge Chen	MiTAC PS/2 machine can not out port 22
	test	PS2, 2		;check ps/2 machine
	jz	NotPS2
	ret
NotPS2:
;;-
	cli
	in	ax,CPUPWRMODE
	mov	bx, ax
	test	ax,UNLOCK	; check CPUPWRMODE reg. locked or not?
	jnz	already_unlock	; unlock, then jump

	mov	al,0		; to enable CPUPWRMODE reg.
	out	CFGSTAT,al
	mov	al,80h
	out	CPUPWRMODE,al
	xor	ah,ah		; AX = 0080h
	out	CPUPWRMODE,ax	; now CPUPWRMODE reg. enabled

already_unlock:
	in	al, CPUPWRMODE
	and	al, 0f0h
	or	al, OBM_CNTL+UNIT_EN+UNLOCK
	out	CPUPWRMODE, al

	mov	dx,_SIGNATURE
	in	ax,dx
	cmp	ah,43h				;4311h -- version B1
	;; 22/3/1993 modeified and add by Y.L Tang
      ;;jne	Not_SL				;4301h -- version A1
	je	CPU386_SL			   ;4301h -- version A1
	mov	dx, 070Ah			;486SL Signature address
	in	ax, dx
	mov	byte ptr ver, 2
	cmp	ah, 44h 			;44xx -- 486SL ID
	je	CPUPWR_Lock
	jmp	Not_SL
	;;

CPU386_SL:
	mov	ah,al
	shr	ah,1
	shr	ah,1
	shr	ah,1
	shr	ah,1		; prepare 1st digit of revision number
	and	al,0fh		; prepare 2nd digit of revision number
	mov	byte ptr ver,ah 		; translate number to version
	mov	byte ptr subver,al		   ; translate number to version

;;----------
;	 mov	 al,0		 ; to enable CPUPWRMODE reg.
;	 out	 CFGSTAT,al
;	 mov	 al,80h
;	 out	 CPUPWRMODE,al
;	 xor	 ah,ah		 ; AX = 0080h
;	 out	 CPUPWRMODE,ax	 ; now CPUPWRMODE reg. enabled
;;-----------
;	 mov	 ax,bx
;	 out	 CPUPWRMODE,ax	 ; Lock CPUPWRMODE register
;	 test	 ax, UNLOCK
;	 jnz	 CPU_not_lock
	;;22/3/1993
 CPUPWR_Lock:
	;;
	or	ax,CPUCNFG_LOCK
	out	CPUPWRMODE,ax	; Lock CPUPWRMODE register
;CPU_not_lock:
CheckIOMode:
	;;22/3/1993 added by Y.L. Tang
	cmp	ver, 2
	je	is_ver_B
	;;
	cmp	ver, 0				;version A
	je	is_ver_A
is_ver_B:
	in	al,CFGSTAT
	mov	cl, al
	test	al,IOCFGOPN
	jz	found_82360SL
;---------
	mov	dx,0fc23h
	in	al,dx
	mov	dx,0f023h
	in	al,dx
	mov	dx,0c023h
	in	al,dx
	mov	dx,00023h
	in	al,dx
	jmp	found_82360SL

Not_SL:
	;;22/3/1993 added by Y.L Tang
    ;;	or	ax,CPUCNFG_LOCK
    ;;	out	CPUPWRMODE,ax	; Lock CPUPWRMODE register
	;;
	jmp	CPU_not_lock

is_ver_A:
	in	al,CFGSTAT
	mov	cl, al
	test	al,IOCFGOPN
	jz	found_82360SL

	mov	dx,0fffbh
	in	al,dx
	mov	dx,0ffffh
	in	al,dx
	mov	dx,0fffdh
	in	al,dx
	mov	dx,0fff9h
	in	al,dx

public found_82360SL
found_82360SL:
;IF	 NDIS_OS2_WKS_driver or ODI_OS2_WKS_driver or Server_386_driver
;	 mov	 ax, PrnPortList
;	 mov	 io_addr, ax
;ELSE
;	 xor	 ax, ax
;	 mov	 es, ax
;	 mov	 ax, es:[408h]
;	 mov	 io_addr, ax
;ENDIF
	cmp	_EBP io_addr, 3bch
	je	PS2Mode
	;; added by Y.L. Tang
	cmp	ver, 2
	je	SetFastMode
	;;
	cmp	ver, 0				;version A
	je	PS2Mode
	cmp	ver, 1				;version B
	jne	SetFastMode
	cmp	subver, 0			;sub version 0
	jne	SetFastMode

PS2Mode:
	mov	al,CFGR2
	out	CFGINDEX,al			;select CFGR2
	in	al,CFGDATA
	mov	ch, al
;	 and	 al, not PS2_EN+SFIO_EN
	or	al, PS2_EN			;enable PS2 Mode
	out	CFGDATA,al			;set CFGR2

	cmp	_EBP io_addr, 0378h
	jne	I278
	mov	ah, 0
	jmp	short IOGet
I278:
	cmp	_EBP io_addr, 0278h
	jne	I3bc
	mov	ah, 20h
	jmp	short IOGet
I3bc:
	mov	ah, 40h
IOGet:
	mov	dx,PORT102
	or	ah, DIR_EN			;enable bidirection
	in	al, dx
	and	al, 09fh
	or	al, ah
	out	dx, al

;	test	ParamFlag, Bit6
;	jnz	ModeSet
;	mov	NICMode, ByteMode	    ;nic mode
;	or	ParamFlag, Bit6
	jmp	short ModeSet

SetFastMode:
	mov	al,CFGR2
	out	CFGINDEX,al			;select CFGR2
	in	al,CFGDATA
	mov	ch,al
	and	al, not PS2_EN+SFIO_EN
	or	al, SFIO_EN			;enable PS2 Mode
	out	CFGDATA,al			;set CFGR2

	out	SFSENABLE, al			;dummy write

	mov	ah, 10h 			;select 378h IRQ7
	cmp	_EBP io_addr, 378h
	je	SFM1
	mov	ah, 20h 			;select 278h IRQ5
SFM1:
;	or	ah, FAST_MODE+EXT_MODE		;enable fast extended mode
;	or	ah, FAST_MODE			;enable fast extended mode
	or	ah, EPPValue

	mov	al, FPP_CNTL
	out	SFINDEX, al			;select FPP_CNTL
	in	al, SFDATA			;get data
	and	al, 4fh
	or	al, ah
;	or	al, EPPValue			;
	out	SFDATA, al
exit0:
;	test	ParamFlag, Bit6
;	jnz	ModeSet
;	mov	NICMode, FastMode	    ;nic mode
;	or	ParamFlag, Bit6
ModeSet:
	mov	_EBP int_level, 0		 ;normal
	or	ParamFlag, Bit4 	    ;INTLevel

;;;	or	ParamFlag, Bit0 	    ;IOBase

SFM2:
	or	PS2, 4				;indicate 360SL

	out	SFSDISABLE, al			;disable 82360SL

	mov	al,CFGR2
	out	CFGINDEX,al
	mov	al,ch
	out	CFGDATA,al

	test	cl, 80h
	jz	not_lock_360SL
	mov	al,IDXLCK
	out	CFGINDEX,al
	mov	al,LOCK360SL
	out	CFGDATA,al

not_lock_360SL:
       mov     al,0	       ; to enable CPUPWRMODE reg.
       out     CFGSTAT,al
       mov     al,80h
       out     CPUPWRMODE,al
       xor     ah,ah	       ; AX = 0080h
       out     CPUPWRMODE,ax   ; now CPUPWRMODE reg. enabled

       mov     ax,bx
       out     CPUPWRMODE,ax   ; Lock CPUPWRMODE register
       test    ax, UNLOCK
       jnz     CPU_not_lock
       or      ax,CPUCNFG_LOCK
       out     CPUPWRMODE,ax   ; Lock CPUPWRMODE register
CPU_not_lock:
	clc
	ret
Detect360SL	endp


;-------------------------------------------------------------------------
;   SetOurVector
;	Set interrupt vectors, for IRQ5 and IRQ7
;
;   Assume
;	ds		DGroup
;-------------------------------------------------------------------------
SetOurVector	proc	near
IF	NDIS_OS2_WKS_driver or ODI_OS2_WKS_driver
	ret
ELSE
	mov	ax, 0
	mov	es, ax
	mov	di, 034h			;es:di IRQ 5 vector
	mov	ax, es:[di]
	mov	OldIrq5, ax			;offset
	mov	ax, es:[di+2]
	mov	OldIrq5[2], ax			;segment
;;	mov	ax, offset CGroup:IRQ5_Vector
	lea	ax, IRQ5_Vector
	mov	es:[di], ax
	mov	es:[di+2], cs

	mov	di, 03ch			;es:di IRQ 7 vector
	mov	ax, es:[di]
	mov	OldIrq7, ax			;offset
	mov	ax, es:[di+2]
	mov	OldIrq7[2], ax			;segment
;;	mov	ax, offset CGroup:IRQ7_Vector
	lea	ax, IRQ7_Vector
	mov	es:[di], ax
	mov	es:[di+2], cs
	ret
ENDIF
SetOurVector	endp


;-------------------------------------------------------------------------
;   RestoreVector
;	Restore interrupt vectors, for IRQ5 and IRQ7
;
;   Assume
;	ds		DGroup
;-------------------------------------------------------------------------
RestoreVector	proc	near
IF	NDIS_OS2_WKS_driver or ODI_OS2_WKS_driver or Server_386_driver
ELSE
	mov	ax, 0
	mov	es, ax
	mov	di, 034h			;es:di IRQ 5 vector
	mov	ax, OldIrq5			;offset
	mov	es:[di], ax
	mov	ax, OldIrq5[2]			;segment
	mov	es:[di+2], ax

	mov	di, 03ch			;set es:di IRQ 7
	mov	ax, OldIrq7			;offset
	mov	es:[di], ax
	mov	ax, OldIrq7[2]			;segment
	mov	es:[di+2], ax
ENDIF
	ret
RestoreVector	endp



;-------------------------------------------------------------------------
;   IRQ5_Vector
;-------------------------------------------------------------------------
IRQ5_Vector	proc	near
IF	NDIS_OS2_WKS_driver or ODI_OS2_WKS_driver
	mov	al, 5
	mov	dl,DevHlp_EOI
	call	devHelper
	clc
	ret
ELSE
	push	ax
	push	bx
	push	dx
	push	ds
	push	cs
	pop	ds

	xor	bx, bx
	loadport
	setport StatusPort
	in	al, dx

	add	IRQ5Count, 1
	mov	al, 20h 		; commands to issue
	out	INT_1EOIR,al		; command to slave
	pop	ds
	pop	dx
	pop	bx
	pop	ax
	iret
ENDIF
IRQ5_Vector	endp



;-------------------------------------------------------------------------
;   IRQ7_Vector
;-------------------------------------------------------------------------
IRQ7_Vector	proc	near
IF	NDIS_OS2_WKS_driver or ODI_OS2_WKS_driver
	mov	al, 7
	mov	dl,DevHlp_EOI
	call	devHelper
	clc
	ret
ELSE
	push	ax
	push	bx
	push	dx
	push	ds
	push	cs
	pop	ds
	xor	bx, bx
	loadport
	setport StatusPort
	in	al, dx

	add	IRQ7Count, 1
	mov	al, 20h 		; commands to issue
	out	INT_1EOIR,al		; command to slave
	pop	ds
	pop	dx
	pop	bx
	pop	ax
	iret
ENDIF
IRQ7_Vector	endp


;-------------------------------------------------------------------------
;   SetEIPValue
;	Determine content of EIP, NIS0,NCTL1,NIS1
;
;   Input
;	Already in NIC Mode
;	DS		DGroup
;	al		de620 model
;-------------------------------------------------------------------------
SetEIPValue	proc	near
	mov	al, EEBuffer.Model
	cmp	al, 3				;CT
	jnz	SEV1
	mov	al, EEBuffer.Media
	or	_EBP EIPRegister, Bit4
	cmp	al, 1				;BNC
	je	SEVRet
	or	_EBP EIPRegister, Bit5
	jmp	SEVRet
SEV1:
	cmp	al, 2				;UTP
	jnz	SEVRet
	or	_EBP EIPRegister, NCTL0+NIS0
SEVRet:
	ret
SetEIPValue	endp


;-------------------------------------------------------------------------
;   WarmStartProcess
;	Do hardware reset let the NIC return to Nibble mode
;
;   Assume
;	ds		DGroup
;	bx		IO Base
;	Already in NIC mode
;-------------------------------------------------------------------------
WarmStartProcess	proc	near
;---------- Reset -----------

	loadport
	setport CmdPort

	cmp	_EBP MPX_exist, 0
	je	No_MPX1
	mov	bh, IRQEN+DS1
	jmp	short WSP1
No_MPX1:
	mov	bh, IRQEN+ICEN+DS1
WSP1:
	or	bh, _EBP OutDirection
	mov	al, bh
	out	dx, al

	call	TimeDelay
	setport DataPort
	mov	al, W_EIP
	out	dx, al				;out EIP CMD
	call	TimeDelay
	or	bh, CS0
	setport CmdPort
	mov	al, bh
	out	dx, al				;command strobe

	call	TimeDelay
	setport DataPort
	mov	al, _EBP EIPRegister
	or	al, HRESET
	out	dx, al				;reset data
	call	TimeDelay
	or	bh, DS0
	mov	al, bh
	setport CmdPort
	out	dx, al				;data strobe
	call	TimeDelay
	xor	bh, DS0+CS0
	mov	al, bh
	out	dx, al				;return to high

	call	TimeDelay

	mov	al, _EBP NIC_Cmd
	out	dx, al
	ret
WarmStartProcess	endp


CheckDelayCounter	proc	near
	loadport
	setport DataPort
	mov	al, W_NPR+0ch
	out	dx, al				;out DIC W_NPR CMD
	setport CmdPort
	mov	bh, _EBP NIC_Cmd
	mov	al, bh
	xor	al, CS0
	out	dx, al				;command strobe

	setport StatusPort
IF	Server_386_driver
	mov	ecx, 0ffffh
ELSE
	xor	cx, cx
ENDIF
	xor	bl, bl
WaitLow:
	in	al, dx
	test	al, READY
	jz	WaitHight
	inc	bl
	loop	WaitLow
	xor	bl, bl				;Can not read low status
	jmp	short GotCounter
WaitHight:
	in	al, dx
	test	al, READY
	loopz	WaitHight
GotCounter:
	mov	_EBP DelayCounter, bl
	mov	al, bh
	setport CmdPort
	out	dx, al
	ret
CheckDelayCounter	endp


;-------------------------------------------------------------------------
;  PresentTest
;	Determine whether the DE620 adapter is existed or not.
;
;  Input:
;	es:di		point to a word list which are printer IOBASE to check
;	cx		count of the list elements
;	ds		DGroup
;
;  Output:
;	cflag		off is success
;	bx		IOBASE found, if success
;
;-------------------------------------------------------------------------
PresentTest	proc	near
	test	ParamFlag, Bit0
	jz	PT_next

	mov	ax, _EBP io_addr

	or	ax, ax
	jz	PT_Error			;not exist

	call	Check_Multiplexer		;find D-Link Multiplexer
	call	Check_Direction 		;check printer direction
	call	Check_de620
	ret

PT_next:
IF	Server_386_driver
	mov	ax, es:[edi]			 ;get IOBASE
ELSE
	mov	ax, es:[di]			;get IOBASE
ENDIF
	or	ax, ax
	jz	PT_Error			;not exist
	mov	_EBP io_addr, ax

	call	Check_Multiplexer		;find D-Link Multiplexer
	call	Check_Direction 		;check printer direction
	call	Check_de620
	jnc	End_PT
IF	Server_386_driver
	add	edi, 2
ELSE
	add	di, 2
ENDIF
	dec	cx
	jnz	PT_next
PT_Error:
	stc					;error
	ret
End_PT:
	clc
	ret
PresentTest	endp


;-------------------------------------------------------------------------
;  Check_de620
;	Determine whether the DE620 adapter is existed or not.
;
;  Input:
;	bx		printer iobase
;	ds		DGroup
;
;  Output:
;	cflag		off is succes
;	all register preserved, except ax
;-------------------------------------------------------------------------
Check_de620	proc	near
IF	Server_386_driver
	push	ecx
ELSE
	push	cx
ENDIF
	push	dx

	test	ParamFlag, Bit6
	jz	CD1
	mov	al, _EBP NICMode
	push	ax
	mov	_EBP NICMode, NIBBLEMODE
CD1:
;;;;;;	call	NIC_Mode			;change to NIC mode
;;;	Mode_NIC
	cmp	_EBP MPX_exist, 0
	jz	No_MPX0
	loadport
	setport CmdPort
	in	al, dx
;;+ 4/28/1993 Andge Chen		for two printer port
	mov	IMRMask, al			;temporary used - IMRMask
;;-
;;+ 4/22/1993 Andge Chen
	and	al, 01fh			;clear direction bits
	or	al, _EBP OutDirection
;;-
	and	al, not SELECTIN
	out	dx, al
	and	al, not INITP
	out	dx, al
	or	al, STROBE+IRQEN
	mov	_EBP NIC_Cmd, al
	out	dx, al
	jmp	short CheckStart

No_MPX0:
	mov	al, INITP+STROBE+IRQEN
	mov	_EBP NIC_Cmd, al
	out	dx, al
CheckStart:

	call	WarmStartProcess		;do hardware reset

	call	CheckDelayCounter		;detect predelay counter

	mov	al, 0A5h			;write 0a5h to CPR
;;	  IDC1INIT	  W_CPR
	mov	ah, W_CPR
	call	IDC1INIT

;;	  IDC2INIT	  R_CPR
	mov	ah, R_CPR
	call	IDC2INIT

	cmp	al, 0A5h			;check return data
	jne	CheckRet
	clc
	jmp	short CD_ret
CheckRet:
;;+ 4/28/1993 Andge Chen	for two printer port must restore CMD port
	loadport
	setport CmdPort
	mov	al, IMRMask			;restore cmd port value
	out	dx, al				;the IMRMask is temporary used
;;-
	dec	_EBP ModeCount			;ModeCount restore
	stc
CD_ret:
	pushf
	test	ParamFlag, Bit6
	jz	CD2
	popf
	pop	ax
	mov	_EBP NICMode, al
	jmp	short CD3
CD2:
	popf
CD3:
	pop	dx
IF	Server_386_driver
	pop	ecx
ELSE
	pop	cx
ENDIF
	ret
Check_de620	endp




;--------------------------------------------------------------------------
;   EnableBidirection
;	PS/2 machine, we can enable the printer port extended mode
;	through POS register, 94h, 102h
;
;   Output
;	ax, dx destroyed
;	bidirection enabled
;--------------------------------------------------------------------------
EnableBidirection	proc	near
	in	al, 94h 			;system board setup register
	slow					;delay
	slow					;delay
	slow					;delay
	mov	ah, al				;save
	and	al, 7fh 			;enable setup mode
	out	94h, al
	slow					;delay
	slow					;delay
	slow					;delay
	mov	dx, 102h			;system board IO byte
	in	al, dx
	slow					;delay
	slow					;delay
	slow					;delay
	and	al, 7fh 			;enable printer extended mode
	out	dx, al
	slow					;delay
	slow					;delay
	slow					;delay
	mov	al, ah
	out	94h, al 			;exit setup mode
	slow					;delay
	slow					;delay
	ret
EnableBidirection	endp



;--------------------------------------------------------------------------
;   Check_Direction
;	Check printer direction control bit and control level
;
;   Input
;	DS		DGroup
;	bx		IO Base
;
;   Output
;	cflag		clear is success
;	InDirection	the content of control bit of input direction
;	OutDirection	the content of control bit of output direction
;	ax, dx destroyed
;--------------------------------------------------------------------------
Check_Direction proc	near
IF	Server_386_driver
	push	ecx
ELSE
	push	cx
ENDIF
	push	si
	push	di


	test	PS2, 2
	jz	not_PS2_machine

	call	EnableBidirection
	mov	_EBP DirBit, 20h		     ;control bit is bit 5
	mov	_EBP InDirection, 20h		     ;1 --> read
	mov	_EBP OutDirection, 0		     ;0 --> write
	mov	_EBP indir,0			     ;now is input direction
	loadport
	setport CmdPort

	in	al, dx
	and	al, 0dfh
	mov	_EBP cmd_value, al		     ;save original content
	jmp	ConfirmCheck

not_PS2_machine:
	loadport
	setport CmdPort
	in	al,dx
	slow
	and	al, 1fh 			;mask bit 7,6,5
	mov	_EBP cmd_value,al		     ;original content
;;;	   mov	   cx,1
	mov	al,0ceh 			;any data
	setport DataPort
next_byte:					;check current direction
	out	dx,al				;out data to data port
	slow					;delay
	slow
	slow
	slow
	mov	ah,al				;save value
	in	al,dx				;get content of data port
	slow
	cmp	al,ah				;compare
	jne	is_indir			;not equal means input dir
;;;	   ror	   al,cl			   ;next data
;;;	   loop    next_byte
	jmp	short is_outdir
is_indir:
	mov	_EBP indir,1			 ;set input direction
is_outdir:
	mov	_EBP DirBit, Bit7+Bit6+Bit5	 ;set direction bit
						;maybe more than one bit will
						;set this case not checked
;--------------------------------------

ConfirmCheck:
	cmp	PS2, 2
	je	yy
	mov	_EBP InDirection, 0e0h
	mov	_EBP OutDirection, 0
yy:
	mov	cl, _EBP cmd_value		 ;original content

	mov	si, 1
	mov	ah, 0ceh			;any data
check_again:
	call	TimeDelay
	loadport
	setport CmdPort
	mov	al, cl				;restore
	or	al, _EBP InDirection
	out	dx, al				;set input direction
	call	TimeDelay
	slow					;delay
	mov	al, ah
	setport DataPort
	out	dx, al
	call	TimeDelay
	in	al,dx
	call	TimeDelay
	cmp	al,ah
	jne	real_is_indir
	jmp	MaybebidirError
real_is_indir:
	setport CmdPort
	mov	al, cl				;restore
	or	al, _EBP OutDirection		 ;set output direction
	out	dx,al
	call	TimeDelay
	setport DataPort
	in	al,dx				;get data
	slow
	cmp	al,ah
	je	real_is_outdir			;comp ok
	jmp	MaybebidirError
real_is_outdir:
	ror	ah, 1
	dec	si				;decrement loop count
	jnz	check_again
	xor	ax, ax				;success
	clc
	jmp	Check_Ret

MaybebidirError:
	cmp	Second_Check, 0
	jne	Not_support_bidir
	mov	Second_Check, 1
	mov	_EBP InDirection, 0
	mov	_EBP OutDirection, 0e0h
	jmp	check_again
;-----------------------------------

Check_Ret:
	mov	Second_Check, 0
	push	ax
	loadport
	setport CmdPort
	mov	al, _EBP cmd_value		     ;get original data
	or	al, _EBP OutDirection
	out	dx, al				;set to original status
	pop	ax

	pop	di
	pop	si
IF	Server_386_driver
	pop	ecx
ELSE
	pop	cx
ENDIF
	ret

Not_support_bidir:
	mov	_EBP InDirection, 0
	mov	_EBP OutDirection, 0
	stc
	jmp	short Check_Ret
Check_Direction endp



;--------------------------------------------------------------------------
;   FindDFR
;	Use binary search method to find a best delay factor
;	Set the TEST bit in SCR, simulate stress case
;
;   Input
;	Already in NIC mode
;	ds		DGroup
;
;   Output
;	cflag		off is success
;	ah		error code
;	al		delay factor returned
;--------------------------------------------------------------------------
public	FindDFR
FindDFR proc	near
	push	bx

IF	Server_386_driver
	push	ecx
ELSE
	push	cx
ENDIF
	push	dx
;---------- set Test bit on in SCR -------------
	mov	al, _EBP SCRRegister		 ;SCR register content
	or	al, TESTON			;set test bit on
;;	  IDC1INIT	  W_SCR
	mov	ah, W_SCR
	call	IDC1INIT
;---------- set delay factor --------------
	mov	al, 0ffh			;255 max value
	mov	Bptr, al
;;	  IDC1INIT	  W_DFR 		  ;command write DFR
;	 mov	 ah, W_DFR
;	 call	 IDC1INIT
	mov	_EBP DFRRegister, al
;;;	call	WDFRINIT
;------------------------------------------
IF	Server_386_driver
	mov	ecx, 4096/2
ELSE
	mov	cx, 4096/2			;length to test
ENDIF
;;	mov	al, ramSize			;get ram size K base
;;	xor	ah, ah
;;	mov	cx, 9				;
;;	shl	ax, cl				;word base
;;	mov	cx, ax				;word count
;;;	call	RAMTest
;;;	jnc	DFRT1
;;;	jmp	DFRRET				;return error
DFRT1:
;---------- set delay factor --------------
	mov	al, 000h			;000 min value
	mov	Fptr, al
;;	  IDC1INIT	  W_DFR 		  ;command write DFR
;	 mov	 ah, W_DFR
;	 call	 IDC1INIT
	mov	_EBP DFRRegister, al
	call	WDFRINIT
;------------------------------------------
IF	Server_386_driver
	mov	ecx, 4096/2
ELSE
	mov	cx, 4096/2			 ;length to test
ENDIF
;;	mov	al, ramSize		    ;get ram size K base
;;	xor	ah, ah
;;	mov	cx, 9				;
;;	shl	ax, cl			       ;word base
;;	mov	cx, ax				;word count
	call	RAMTest
	mov	al, 0				;return DFR
	jc	DFR2
	jmp	DFRGood
DFR2:
;---------- binary search ----------------
Search1:
	mov	al, Bptr
	sub	al, Fptr
	shr	al, 1				;devide 2
	or	al, al
	jnz	NW8
	mov	al, Bptr		    ;Bptr is the best solution
	jmp	DFRGood
NW8:
	add	al, Fptr
	mov	Cptr, al		    ;new Fptr value

;---------- set delay factor --------------
;;	  IDC1INIT	  W_DFR
;	 mov	 ah, W_DFR
;	 call	 IDC1INIT
	mov	_EBP DFRRegister, al
	call	WDFRINIT
;---------- RAM Test ----------------------
IF	Server_386_driver
	mov	ecx, 4096/2
ELSE
	mov	cx, 4096/2			 ;length to test
ENDIF
;;	mov	al, ramSize		    ;get ram size K base
;;	xor	ah, ah
;;	mov	cx, 9				;
;;	shl	ax, cl			       ;word base
;;	mov	cx, ax				;word count
	call	RAMTest
	jc	NW9				;
	mov	ah, Cptr		    ;Success
	mov	Bptr, ah		    ;new Bptr
	jmp	Search1
NW9:
	mov	ah, Cptr		    ;Fail
	mov	Fptr, ah		    ;new Fptr
	jmp	Search1
;-----------------------------------------
DFRGood:					;al=delay factor value
	clc
DFRRet:
	pushf
	push	ax
	mov	al, _EBP SCRRegister		 ;set Normal mode
;;	  IDC1INIT	  W_SCR
	mov	ah, W_SCR
	call	IDC1INIT
	pop	ax

	popf					;return flag
	pop	dx
IF	Server_386_driver
	pop	ecx
ELSE
	pop	cx
ENDIF
	pop	bx
	ret
FindDFR endp


RAMTestW	proc	near
;	mov	al, _EBP SCRRegister
;;	  IDC1	  W_SCR
;	mov	ah, W_SCR
;	call	IDC1

	mov	bl, 0A5h			;write 0a5h to CPR
IF	Server_386_driver
	mov	ecx, 3
ELSE
	mov	cx, 3
ENDIF
RTW1:
	push	bx
	mov	al, bl
;;	  IDC1	      W_CPR
	mov	ah, W_CPR
	call	IDC1

	call	TimeDelay

	mov	ah, R_CPR
	call	IDC2
	pop	bx
	cmp	al, bl
	je	RTWRet
	stc
	ret
RTWRet:
	shr	bl, 1
	loop	RTW1
	clc
	ret
RAMTestW	endp




;--------------------------------------------------------------------------
;   RAMTest
;	Do RAM test, remote write 0, 1, 2 ... then remote read and compare
;	do it again but remote write ffff, fffe, fffd ...
;
;   Input
;	Already in NIC mode
;	ds		DGroup
;	cx		size of RAM, word base
;	bx		io base
;
;   Output
;	cflag		off is success
;--------------------------------------------------------------------------
public RAMTest
RAMTest proc	near
IFDEF ps
print R
ENDIF
	push	si
	push	cx

	mov	al, RW0 			;remote write buffer 0
;;	  WCR
	call	WCR

;------
	push	cx
	mov	ax, ds
	mov	es, ax
	xor	ax, ax
MT1:
IF	Server_386_driver
	lea	edi, RAMTestBuffer		 ;get buffer offset
ELSE
	lea	di, RAMTestBuffer		;get buffer offset
ENDIF
	cmp	cx, 128 			;buffer size
	jb	MT2
	sub	cx, 128
	push	cx				;save test size
	mov	cx, 128
	mov	bx, cx
	jmp	MT3
MT2:
	xor	bx, bx
	push	bx				;save test size
	mov	bx, cx
MT3:
	stosw
	inc	ax
;;;	   loop    MT3
	dec	cx
	jnz	MT3

;;;	mov	di, ax				;temporary save
	push	ax
IF	Server_386_driver
	lea	esi, RAMTestBuffer
ELSE
	lea	si, RAMTestBuffer
ENDIF
	mov	cx, bx
	shl	cx, 1				;transfer to byte size
	call	RemoteWrite
	pop	ax
	pop	cx
;;;	   jcxz    MT4
	or	cx, cx
	jz	MT4
	jmp	MT1
MT4:
	pop	cx				;restore test size

	call	WDUMMY
;---------- remote read -------------
	mov	al, RRN 			;set remote read
;;	  WCR					  ;issue command register
	call	WCR
;---------- set remote starting address --------
	cmp	_EBP NICMode, BYTEMODE
	je	TT7
	jmp	NoDirChg1
TT7:
;;	  WDUMMY				  ;turn direction of de620
	call	WDUMMY
public NoDirChg1
NoDirChg1:
	push	cx
	xor	ax, ax				;RSA1, RSA0 = 0
;;	  WRSA
	call	WRSA
	pop	cx

;----------- read data and compare -------
	push	cx
	xor	si, si
MT5:
	cmp	cx, 128
	jb	MT6
	sub	cx, 128
	push	cx				;save test size
	mov	cx, 128
	jmp	MT7
MT6:
	xor	ax, ax
	push	ax				;save test size
MT7:
	push	cx
	shl	cx, 1
IF	Server_386_driver
	lea	edi, RAMTestBuffer
ELSE
	lea	di, RAMTestBuffer
ENDIF
	call	GetRxData
	pop	cx

IF	Server_386_driver
	lea	edi, RAMTestBuffer
ELSE
	lea	di, RAMTestBuffer
ENDIF
MT8:
IF	Server_386_driver
	cmp	si, es:[edi]
ELSE
	cmp	si, es:[di]
ENDIF
	je	MT9
	pop	cx				;remove cx
	pop	cx
	jmp	RAMTestError
MT9:
IF	Server_386_driver
	add	edi, 2
ELSE
	add	di, 2
ENDIF
	inc	si
;;;	   loop    MT8
	dec	cx
	jnz	MT8

	pop	cx				;restore test size
;;;	   jcxz    MT10
	or	cx, cx
	jz	MT10
	jmp	short MT5
MT10:
	pop	cx				;restore size

;;	  WDUMMY				  ;change direction of de620
	call	WDUMMY
IFDEF	ps
print a
ENDIF

NoDirChg3:
;----------- second check -----------

	mov	al, RW0 			;remote write buffer 0
;;	  WCR
	call	WCR

	push	cx
	mov	ax, ds
	mov	es, ax
	mov	ax, cx
MT11:
IF	Server_386_driver
	lea	edi, RAMTestBuffer		 ;get buffer offset
ELSE
	lea	di, RAMTestBuffer		;get buffer offset
ENDIF
	cmp	cx, 128 			;buffer size
	jb	MT12
	sub	cx, 128
	push	cx				;save test size
	mov	cx, 128
	mov	bx, cx
	jmp	MT13
MT12:
	xor	bx, bx
	push	bx				;save test size
	mov	bx, cx
MT13:
	stosw
	dec	ax
;;;	   loop    MT13
	dec	cx
	jnz	MT13

;;;	mov	di, ax				;temporary save
	push	ax
IF	Server_386_driver
	lea	esi, RAMTestBuffer
ELSE
	lea	si, RAMTestBuffer
ENDIF
	mov	cx, bx
	shl	cx, 1
	call	RemoteWrite
;;;	mov	ax, di
	pop	ax
	pop	cx
;;;	   jcxz    MT14
	or	cx, cx
	jz	MT14
	jmp	MT11
MT14:
	pop	cx				;restore test size

	call	WDUMMY
;---------- remote read -------------
IFDEF	OLD
	mov	al, RR				;set remote read
ELSE
	mov	al, RRN 			;set remote read
ENDIF
;;	  WCR
	call	WCR
;---------- set remote starting address --------
	cmp	_EBP NICMode, BYTEMODE
	je	TT5
	jmp	NoDirChg2
TT5:
;;	  WDUMMY
	call	WDUMMY
NoDirChg2:
	push	cx
	xor	ax, ax				;RSA1, RSA0 = 0
;;	  WRSA
	call	WRSA
	pop	cx
;----------- read data and compare -------
	push	cx
	mov	si, cx
MT15:
	cmp	cx, 128
	jb	MT16
	sub	cx, 128
	push	cx				;save test size
	mov	cx, 128
	jmp	MT17
MT16:
	xor	ax, ax
	push	ax				;save test size
MT17:
	push	cx
	shl	cx, 1
IF	Server_386_driver
	lea	edi, RAMTestBuffer
ELSE
	lea	di, RAMTestBuffer
ENDIF
	call	GetRxData
	pop	cx

IF	Server_386_driver
	lea	edi, RAMTestBuffer
ELSE
	lea	di, RAMTestBuffer
ENDIF
MT18:
IF	Server_386_driver
	cmp	si, es:[edi]
ELSE
	cmp	si, es:[di]
ENDIF
	je	MT19
	pop	cx				;remove cx
	pop	cx
	jmp	RAMTestError
MT19:
IF	Server_386_driver
	add	edi, 2
ELSE
	add	di, 2
ENDIF
	dec	si
;;;	   loop    MT18
	dec	cx
	jnz	MT18
	pop	cx				;restore test size
;;;	   jcxz    MT20
	or	cx, cx
	jz	MT20
	jmp	short MT15
MT20:
	pop	cx				;restore size

;;	  WDUMMY				  ;change direction of de620
	call	WDUMMY
IFDEF	ps
print A
ENDIF
NoDirChg4:
IFDEF ps
print t
ENDIF
	clc
	pop	cx
	pop	si
	ret
RAMTestError:
IFDEF	ps
print E
ENDIF
;;	  WDUMMY				  ;change direction of de620
;;;;	call	WDUMMY
	test	_EBP NICMode, FASTMODE
	jnz	WWD1

	loadport
	setport StatusPort
;	 RDYWait
	call	TimeDelay

	setport CmdPort
	mov	al, _EBP NIC_Cmd
	out	dx, al
	call	TimeDelay

	mov	al, W_DUMMY
	setport DataPort
;	 OutCommand
	out	dx, al
	call	TimeDelay
	setport CmdPort
	mov	al, _EBP NIC_Cmd
	xor	al, CS0
	out	dx, al

	setport StatusPort
;	 RDYWait
	call	TimeDelay

	setport CmdPort
	mov	al, _EBP NIC_Cmd
	out	dx, al
	jmp	WWD2
WWD1:
	mov	al, W_DUMMY
	OutCommandF
WWD2:
;-----
IFDEF ps
print s
ENDIF
	stc
	pop	cx
	pop	si
	ret

RAMTest endp



;--------------------------------------------------------------------------
;   IRQCheck
;	Find IRQ # of printer port
;
;   Input
;	In printer mode
;	DS		DGroup
;	io_addr 	word variable of io base inDGroup
;
;   Output
;	cflag		off is success
;	irq		irq number
;	SCRRegister	IRQ inverter bit set
;	ax destroyed
;
;--------------------------------------------------------------------------

public	IRQCheck
IRQCheck	proc	near
	push	bx
	push	cx
	push	dx

	mov	_EBP int_level, 0
IRQCheck1:
	in	al, INT_1IMR		;get 8259 channel mask
	jmp	$+2
	mov	IMRMask, al
	mov	bl, 0a0h			;mask IRQ 5, 7
	mov	ah, bl
	or	al, bl
	out	INT_1IMR, al			;mask
	jmp	$+2
	jmp	$+2
	jmp	$+2
	mov	al, 0ah 			;read IRR in next pulse
	out	INT_1EOIR, al
	jmp	$+2				;delay
	jmp	$+2
	jmp	$+2
	jmp	$+2

	in	al, INT_1EOIR			;get IRR
	and	al, bl				;get bit 7, 5
	xor	al,bl				;inverse bit 7,5
	and	ah,al				;bit set means no IRQ request
	jnz	IRQ__OK 			;nonzero means irq empty
	jmp	irq_err 			;Is all IRQ channels being used?

IRQ__OK:
	push	ax				;ah=which one is empty IRQ
	push	bx				;bl=0a0h

	call	TimeDelay
	mov	al, _EBP SCRRegister
	or	al, _EBP int_level
	xor	al, IRQINV			;inverse inverter
	mov	ah, W_SCR
	test	_EBP NICMode, FASTMODE
	jz	IRQID4
	jmp	IRQID1
IRQID4:
	loadport
	push	ax
	mov	al, ah
	setport DataPort
	out	dx, al				;SCR CMD
	call	TimeDelay
	setport CmdPort
	mov	al, _EBP NIC_Cmd
	or	al, IRQEN
	xor	al, CS0
	out	dx, al				;CS0

	call	TimeDelay
	pop	ax
	push	ax
	setport DataPort
	out	dx, al				;data
	call	TimeDelay
	setport CmdPort
	xor	_EBP NIC_Cmd, DS0 or DS1
	mov	al, _EBP NIC_Cmd
	or	al, IRQEN
	out	dx, al				;data strobe
	pop	ax
	jmp	IRQID2
IRQID1:
	push	ax
	mov	al, ah
	OutCommandF
	pop	ax
	loadport
	setport DataStrobe0
	out	dx, al
IRQID2:

	call	TimeDelay

	pop	bx
	pop	ax

	mov	cx, 10
;loop1:
	mov	al, 0ah
	out	INT_1EOIR, al			;select ISR in next input
	in	al, 61h 			;delay
	in	al, 61h 			;delay
	in	al, 61h 			;delay
	in	al, INT_1EOIR
;	 slow				       ;Why loop ?
;	 loop	 loop1

	and	al, bl				;get bit 7,5
	and	ah, al				;get which bit on
	jz	irq_err 			;zero is no IRQ
	mov	bl, al
	jmp	short rst_imask

irq_err:
	mov	al, IMRMask
	out	INT_1IMR,al			;restore original content
	cmp	byte ptr Second_Check, 1	;second time test
	je	is_second_check

	mov	byte ptr Second_Check, 1	;set second time flag
	mov	byte ptr _EBP int_level, IRQINV      ;inverter on
	jmp	IRQCheck1

rst_imask:
	mov	al, IMRMask			;original content
	out	INT_1IMR, al			;restore
calcIRQ:
	mov	cx, 8
rolirq:
	ror	bl, 1				;shift bit 0 to cflag
	jc	setirq1
;;;	   loop    rolirq
	dec	cx
	jnz	rolirq
	xor	bx, bx				;data base
	jmp	irq_err
setirq1:
	xor	bx, bx				;data base
	dec	cl
	not	cl
	and	cl, 7
	mov	_EBP irq, cl			 ;got irq number

IRQ_set_already:
	mov	al, _EBP irq
	or	al, EOI_SPECIFIC
	out	INT_1EOIR, al			;send EOI
;---------------
; 4/10/92 Andge Chen	   Inverse the result
;	xor	int_level, IRQINV
;------------------
	mov	al, _EBP int_level
	or	_EBP SCRRegister, al		     ;set interrup inverter
	clc
	pop	dx
	pop	cx
	pop	bx
	ret
is_second_check:
	stc
	pop	dx
	pop	cx
	pop	bx
	ret
IRQCheck	endp




;-------------------------------------------------------------------------
;  SetDefault
;	Specified default value is from EEPROM
;-------------------------------------------------------------------------
public	SetDefault
SetDefault	proc	near
	mov	DefaultFlag, 0
	ret
SetDefault	endp



;-------------------------------------------------------------------------
;   SetIOBase
;	User specify IO Base value
;
;   Input
;	DS		DGroup
;	ax		IO Base
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	SetIOBase
SetIOBase     proc    near
	mov	_EBP io_addr, ax		     ;store iobase
	or	ParamFlag, Bit0 		;set flag
	ret
SetIOBase     endp



;-------------------------------------------------------------------------
;   SetIRQ
;	User specify IRQ number
;
;   Input
;	DS		DGroup
;	al		IRQ Number
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	SetIRQ
SetIRQ	      proc    near
	mov	_EBP irq, al			     ;store irq number
	or	ParamFlag, Bit1 		;set flag
	ret
SetIRQ	      endp



;-------------------------------------------------------------------------
;   SetROMBase
;	User specify ROM Base value
;
;   Input
;	DS		DGroup
;	ax		ROM Base
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	SetROMBase
SetROMBase     proc    near
	ret
SetROMBase     endp



;-------------------------------------------------------------------------
;   SetNodeID
;	User specify Node address
;
;   Input
;	DS		DGroup
;	es:si		point to node address
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	SetNodeID
SetNodeID     proc    near
	push	cx
	push	ax
IF	Server_386_driver
	push	edi
	push	esi
ELSE
	push	di
	push	si
ENDIF
	push	ds
	push	es
	push	bx

	or	ParamFlag, Bit3
	mov	ax, ds
	push	es
	pop	ds
	mov	es, ax
IF	Server_386_driver
	mov	ecx, 6
;;	mov	esi, edi		; by ted tai
	lea	edi, _EBP DNodeAddress
ELSE
	mov	cx, 6
;;	mov	si, di			; by ted tai
	lea	di, DNodeAddress
ENDIF
	rep	movsb

	pop	bx
	pop	es
	pop	ds
IF	Server_386_driver
	pop	esi
	pop	edi
ELSE
	pop	si
	pop	di
ENDIF
	pop	ax
	pop	cx
	ret
SetNodeID     endp



;-------------------------------------------------------------------------
;   SetINTLevel
;	User specify interrupt level
;
;   Input
;	DS		DGroup
;	al		Interrupt Level:  0 -- normal
;					  1 -- inverse
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	SetINTLevel
SetINTLevel   proc    near

	or	ParamFlag, Bit4
	mov	_EBP int_level, 0
	or	al, al
	jz	NW10
	mov	_EBP int_level, IRQINV
NW10:
	ret
SetINTLevel   endp



;-------------------------------------------------------------------------
;   SetWDFR
;	User specify delay factor value
;
;   Input
;	DS		DGroup
;	al		delay factor
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	SetWDFR
SetWDFR        proc    near
	or	ParamFlag, Bit5
	mov	_EBP DFRRegister, al
	ret
SetWDFR        endp



;-------------------------------------------------------------------------
;   SetRDFR
;	User specify delay factor value
;
;   Input
;	DS		DGroup
;	al		delay factor
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	SetRDFR
SetRDFR        proc    near
	or	ParamFlag, Bit8
	mov	_EBP RDFRRegister, al
	ret
SetRDFR        endp


;-------------------------------------------------------------------------
;   SetNICMode
;	User specify NIC Mode
;
;   Input
;	DS		DGroup
;	al		NIC Mode    0 -- nibble mode
;				    10h - Byte mode
;				    20h - Fast mode
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	SetNICMode
SetNICMode    proc    near
	or	ParamFlag, Bit6
	mov	_EBP NICMode, al
	ret
SetNICMode    endp


;-------------------------------------------------------------------------
;   Set360SL
;	User specify 82360SL Special Feature( enable fast mode )
;
;   Input
;	DS		DGroup
;	al		NIC Mode    1 -- Not enable any feature
;				    0 -- Enable 360SL Spwcial Feature
;
;   Output
;   Modified Registers
;	None
;-------------------------------------------------------------------------
public	Set360SL
Set360SL      proc    near
	or	al, al
	jz	S360
	or	ParamFlag, Bit7
S360:
	ret
Set360SL      endp



;-------------------------------------------------------------------------
;   check_PS2
;	check MCA machine
;
;-------------------------------------------------------------------------
check_ps2	proc	near
IF	NDIS_OS2_WKS_driver or ODI_OS2_WKS_driver or Server_386_driver
ELSE
	mov	ax,0c400h
	int	15h
	jc	not_PS2
	or	PS2,2
ENDIF
not_PS2:
	ret
check_ps2	endp



;
; ReadEEPROMSetting
;
;
;    Description :
;
;	The routine read the data in EEPROM from NIC via port IOBase+1Eh.
;
;    Entry :
;	Already in NIC mode
;	DS    -> DGroup
;	ES:DI -> point to a NICSetting type buffer for saving NIC setting
;
;    Output :
;
;	if AX=0   EEPROM data is read successfully
;
;	   AdapterAddress,IOAddress,IRQ,ROMAddress,MediaType will be save
;	   in the buffer pointed by ES:DI
;
;	if AX=ErrEEPROMRead		EEPROM reading error or Card not found
;	if AX=ErrEEPROMChecksum 	EEPROM checksum incorrect
;
;    Modified :
;	AX
;

ReadEEPROMSetting  proc
	push	bx
	push	cx
	push	si

;	 push	 dx
;	 mov	 si, 04ch/2
;	 mov	 cx, 13
;ChkSum:					 ;check sum test
;	 call	 ReadAWord
;	 add	 dl, al
;	 add	 dl, ah
;	 inc	 si
;	 loop	 ChkSum
;	 or	 dl, dl
;	 pop	 dx
;	 jnz	 ChecksumError

IF	Server_386_driver
	mov	es:[edi].Major_Version,1 ;set Version 1.0
	mov	es:[edi].Minor_Version,0
ELSE
	mov	es:[di].Major_Version,1 ;set Version 1.0
	mov	es:[di].Minor_Version,0
ENDIF
	mov	si, 02ah		;si -> EEPROM address
	call	ReadAWord		;read word 0
IFDEF	PROTECT
	cmp	al, NodeIDByte0 	;check adapter byte 0
	jne	EEPROMReadError
	cmp	ah, NodeIDByte1 	;chech adapter byte 1
	jne	EEPROMReadError
ENDIF
IF	Server_386_driver
	mov	Word ptr es:[edi].NodeID,ax; save adapter address byte 0 and byte 1
ELSE
	mov	Word ptr es:[di].NodeID,ax; save adapter address byte 0 and byte 1
ENDIF
	inc	si			;increase EEPROM address
	call	ReadAWord		;read word 1
IFDEF	PROTECT
	cmp	al, NodeIDByte2       ;check adapter byte 2
	je	CardFound
EEPROMReadError:
	mov	ax,ErrEEPROMRead
	jmp	ReadNICSetting_ret
ENDIF

CardFound:
IF	Server_386_driver
	mov	Word ptr es:[edi].NodeID+2,ax; save adapter address byte 2 and byte 3
ELSE
	mov	Word ptr es:[di].NodeID+2,ax	 ; save adapter address byte 2 and byte 3
ENDIF
	inc	si			;increase EEPROM address
	call	ReadAWord		;read word 2
IF	Server_386_driver
	mov	Word ptr es:[edi].NodeID+4,ax ; save adapter address
ELSE
	mov	Word ptr es:[di].NodeID+4,ax ; save adapter address
ENDIF

	mov	si, 05ah/2			;IRQ, RAM size
	call	ReadAWord
IF	Server_386_driver
	mov	es:[edi].RAM_Size, ah
	mov	es:[edi].IRQValue, al
ELSE
	mov	es:[di].RAM_Size, ah
	mov	es:[di].IRQValue, al
ENDIF

	mov	si, 05ch/2			;hardware model
	call	ReadAWord
IF	Server_386_driver
	mov	es:[edi].Model, al		 ;get model
	mov	es:[edi].RdDelayFactor, ah
ELSE
	mov	es:[di].Model, al		;get model
	mov	es:[di].RdDelayFactor, ah
ENDIF

	mov	si, 04ch/2			;NPR DFR
	call	ReadAWord
IF	Server_386_driver
	mov	es:[edi].WtDelayFactor, ah	 ;get delay factor
ELSE
	mov	es:[di].WtDelayFactor, ah	;get delay factor
ENDIF

	mov	si, 050h/2			;EPR, SCR
	call	ReadAWord
IF	Server_386_driver
	mov	es:[edi].SCRValue, ah
ELSE
	mov	es:[di].SCRValue, ah
ENDIF

	mov	si, 05eh/2
	call	ReadAWord
IF	Server_386_driver
	mov	es:[edi].Media, al
	mov	es:[edi].Auto, ah
ELSE
	mov	es:[di].Media, al
	mov	es:[di].Auto, ah
ENDIF

	xor	ax, ax
ReadNICSetting_ret:
	pop	si
	pop	cx
	pop	bx
	ret
ChecksumError:
	mov	ax,ErrEEPROMChecksum
	jmp	short ReadNICSetting_ret
ReadEEPROMSetting  endp

;ReadAWord
;
; Input :
;	   si	EEPROM address
;
; Output :
;	   ax : the word read
;
; Modified :
;	   None
;

ReadAWord	proc
	push	bx
	push	cx
	call	SetSendState
	mov	ax,si
	or	ax,110000000B			;Read a word command
	mov	cx,09h
	call	SendBitsToChip
	mov	cx,10h
	xor	ax,ax
	xor	bx,bx
sa0:
	shl	bx,1
	call	send_0				;clocking EEPROM
	call	ReadBit
	and	al,01h
	or	bl,al
;;;	   loop    sa0
	dec	cx
	jnz	sa0
	call	ResetSendState
	mov	ax,bx
	pop	cx
	pop	bx
	ret
ReadAWord	endp

;SendBitsToChip
;
; Input :
;	 TargetChip -> EEPROMChip
;		    -> DL2516Chip
;	  ax -> data to be sent, the data is sent from High bit
;	  cx -> number of bits to be sent
;	  dx -> EEPROM IO Address
;
; Output :
;	  None
;
; Modified :
;	  None
;

SendBitsToChip		proc
	push	ax
	push	bx				;send data in AX for CL times
	push	cx
	mov	bx,cx				; save cx
	mov	ch,10h				;
	sub	ch,cl				; mov meaningful bits to high
	mov	cl,ch				;  bits of AX
	shl	ax,cl				;
	mov	cx,bx				; restore cx
n_d_b:
	mov	bx,ax
	and	bx,8000h			;Hi bit first
	cmp	bx,000h
	je	SendBit0
	call	send_1
	jmp	SendNextBit
SendBit0:
	call	send_0
SendNextBit:
	shl	ax,1
;;;	   loop    n_d_b
	dec	cx
	jnz	n_d_b
	pop	cx
	pop	bx
	pop	ax
	ret
SendBitsToChip		endp

;SetSendState
;
;  Input :
;  Output :
;     None
;  Modified :
;     None
;

SetSendState	proc				;for send_begin state
	push	ax				;cs  __--
	mov	al,00h				;di  ____
						;sk  _--_
	call	send
	mov	al,01h
	call	send
	mov	al,05h
	call	send
	mov	al,04h
	call	send
	pop	ax
	ret
SetSendState	endp

;
;ResetSendState
;
; input :
; output :
;	None
; Modified :
;	None
;

ResetSendState	proc				;cs	____
	push	ax				;di	____
	mov	al,00h				;sk	_--_
	call	send
	mov	al,01h
	call	send
	call	send
	mov	al,00h
	call	send
	pop	ax
	ret
ResetSendState	endp

;Send_1
;
; input :
; output :
;	None
; modified :
;	None
;

send_1	proc
	push	ax				;cs	----
	mov	al,06h				;di	----
	call	send				;sck	_--_
	mov	al,07h
	call	send
	call	send
	mov	al,06h
	call	send
	pop	ax
	ret
send_1	endp

;Send_0
;
; input :
; output :
;	None
; modified :
;	None
;

send_0	proc
	push	ax				;cs	----
	mov	al,04h				;di	____
	call	send				;sck	_--_
	mov	al,05h
	call	send
	call	send
	mov	al,04h
	call	send
	pop	ax
	ret
send_0	endp


;send
;
; Input :
;	 al -> data to be sent
; Output:
;	 None
; Modified :
;	 None
;

send	proc					;send AL data to port IOAddress
	push	ax
	push	bx
	push	cx
	push	dx
	or	al, _EBP EIPRegister		     ;or EIP register content
;;	  IDC1INIT	  W_EIP
	mov	ah, W_EIP
	call	IDC1INIT
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	ret
send	endp




;inputbit
;
; Input :
; Output:
;	 al	bit 0 is return data
; Modified :
;	 ax
;

ReadBit proc	near				;send AL data to port IOAddress
	push	bx
	push	cx
	push	dx
;;	  IDC2INIT	  R_STS
	mov	ah, R_STS
	call	IDC2INIT
	mov	cl, 7
	shr	al, cl
	pop	dx
	pop	cx
	pop	bx
	ret
ReadBit endp


TimeDelay	proc	near
	push	ax
	push	cx
	mov	cx, 100
TD1:
	in	al, 61h
	jmp	$+2
	jmp	$+2
;;;	   loop    TD1
	dec	cx
	jnz	TD1
	pop	cx
	pop	ax
	ret
TimeDelay	endp


IF    Server_386_driver
InitData	proc	near
;;	  mov	  _EBP io_addr, 378h
	mov	_EBP irq, 7
	mov	_EBP DFRRegister, 0ffh
	mov	_EBP RDFRRegister, 0ffh
	mov	_EBP NICMode, NIBBLEMODE
	mov	_EBP SCRRegister, NIBBLEMODE+INTON+SLEEP
	mov	_EBP TCRRegister, 02h ; Added for support (921021)
	mov	_EBP EIPRegister, Bit6

	mov	_EBP EPRRegister, 0
	mov	_EBP SPRRegister, 12

	mov	_EBP Prn_Cmd, 01ch
	mov	_EBP NIC_Cmd, DS1

	mov	_EBP InDirection, 020h
	mov	_EBP OutDirection, 0

	mov	_EBP StartPacketRx, DYES
	mov	_EBP DisableTx, DNO
	ret
InitData	endp
ENDIF
